@@ -28,9 +28,6 @@ def _should_ignore(fd_name: str, patterns: Sequence[str]) -> bool:
 class FileDescriptorSetGenerator(abc.ABC):
     """Base class that implements fixing imports."""
 
-    def __init__(self, fdset_generator_binary: str) -> None:
-        self.fdset_generator_binary = fdset_generator_binary
-
     @abc.abstractmethod
     def generate_file_descriptor_set_bytes(self) -> bytes:
         """Generate the bytes of a `FileDescriptorSet`"""
@@ -44,7 +41,6 @@ class FileDescriptorSetGenerator(abc.ABC):
         exclude_imports_glob: Sequence[str],
     ) -> None:
         """Fix imports from protoc/buf generated code."""
-        rewriters = {}
         fdset = FileDescriptorSet.FromString(self.generate_file_descriptor_set_bytes())
 
         for fd in fdset.file:
@@ -53,7 +49,7 @@ class FileDescriptorSetGenerator(abc.ABC):
                 continue
 
             fd_name = _remove_proto_suffix(name)
-            rewriters[fd_name] = rewriter = ASTImportRewriter()
+            rewriter = ASTImportRewriter()
             # services live outside of the corresponding generated Python
             # module, but they import it so we register a rewrite for the
             # current proto as a dependency of itself to handle the case
@@ -77,7 +73,7 @@ class FileDescriptorSetGenerator(abc.ABC):
                 except FileNotFoundError:
                     pass
                 else:
-                    new_code = rewriters[fd_name].rewrite(raw_code)
+                    new_code = rewriter.rewrite(raw_code)
                     overwrite_callback(python_file, new_code)
 
         if create_package:
@@ -90,13 +86,15 @@ class FileDescriptorSetGenerator(abc.ABC):
 
 
 class Protoc(FileDescriptorSetGenerator):
+    """Generate the FileDescriptorSet using `protoc`."""
+
     def __init__(
         self,
         protoc_path: str,
         proto_files: Iterable[Path],
         proto_paths: Iterable[Path],
     ) -> None:
-        super().__init__(protoc_path)
+        self.protoc_path = protoc_path
         self.proto_files = list(proto_files)
         self.proto_paths = list(proto_paths)
 
@@ -105,7 +103,7 @@ class Protoc(FileDescriptorSetGenerator):
             filename = Path(f.name)
             subprocess.check_output(
                 [
-                    self.fdset_generator_binary,
+                    self.protoc_path,
                     "--include_imports",
                     f"--descriptor_set_out={filename}",
                     *map("--proto_path={}".format, self.proto_paths),
@@ -120,10 +118,15 @@ class Protoc(FileDescriptorSetGenerator):
 
 
 class Buf(FileDescriptorSetGenerator):
+    """Generate the FileDescriptorSet using `buf`."""
+
+    def __init__(self, buf_path: str) -> None:
+        self.buf_path = buf_path
+
     def generate_file_descriptor_set_bytes(self) -> bytes:
         return subprocess.check_output(
             [
-                self.fdset_generator_binary,
+                self.buf_path,
                 "build",
                 "--as-file-descriptor-set",
                 "--exclude-source-info",
@@ -131,3 +134,13 @@ class Buf(FileDescriptorSetGenerator):
                 "-",
             ]
         )
+
+
+class Raw(FileDescriptorSetGenerator):
+    """Generate the FileDescriptorSet using user-provided bytes."""
+
+    def __init__(self, fdset_bytes: bytes) -> None:
+        self.fdset_bytes = fdset_bytes
+
+    def generate_file_descriptor_set_bytes(self) -> bytes:
+        return self.fdset_bytes
